# faspio Gateway

The faspio Gateway acts as a transport layer proxy between TCP and FASP.

## Usage

### Single Gateway Usage

In the primary use case, the gateway is placed in front of a regular TCP Server, either on the server host itself or in the close proximity (same LAN).

Any request to the gateway (over FASP) is then proxied to the real server over TCP. 

On the client side, any of the faspio SDKs or faspio Agent can be used.

### Gateway Client/Server Usage

In this configuration, two faspio Gateways are used to bridge one (or several) TCP connection(s) from TCP client(s) to a TCP server over FASP

```
   +-----------+         +--------------+                      +---------------+           +-----------+
   |           |         |              |                      |               |           |           |
   |           |         |              |                      |               |           |           |
   | Client(s) |   TCP   |  faspio GW1 |         FASP         |  faspio GW2  |    TCP    |   Server  |
   |           |-------->=              |--------------------->=               |---------->=           |
   |           |         |              |                      |               |           |           |
   +-----------+         +--------------+                      +---------------+           +-----------+
```

Given a Server listening on port 12345, configure the following:

Simply point your client(s) to the faspio GW1 IP and port 12345
	
 #### faspio GW1 Config
 ```toml
[[bridge]]
    [bridge.local]
	    protocol = "tcp"
            host = "0.0.0.0"
		    port = 12345

    [bridge.forward]
	    protocol = "fasp"
		    host = "GW2"
		    port = 12345
 ```

#### faspio GW2 Config
 ```toml
[[bridge]]
    [bridge.local]
        protocol = "fasp"
            host = "0.0.0.0"
            port = 12345

    [bridge.forward]
        protocol = "tcp"
            host = "Server"
            port = 12345
 ```

### Gateway Server/Server Usage

In this mode, each server initiates a connection to the other

```
   +-----------+         +--------------+                      +---------------+           +-----------+
   |           |         |              |                      |               |           |           |
   |           =<--------|              =<---------------------|               =<----------|           |
   |  Server1  |   TCP   |  faspio GW1 |         FASP         |  faspio GW2  |    TCP    |  Server2  |
   |           |-------->=              |--------------------->=               |---------->=           |
   |           |         |              |                      |               |           |           |
   +-----------+         +--------------+                      +---------------+           +-----------+
```

 #### faspio GW1 Config
 ```toml
[[bridge]]
    name = "Outbound"
    [bridge.local]
        protocol = "tcp"
            host = "0.0.0.0"
            port = 12345

    [bridge.forward]
        protocol = "fasp"
            host = "GW2"
            port = 12345

[[bridge]]
    name = "Inbound"
    [bridge.local]
        protocol = "fasp"
            host = "0.0.0.0"
            port = 54321

    [bridge.forward]
        protocol = "tcp"
            host = "Server1"
            port = 54321
 ```

#### faspio GW2 Config
 ```toml
[[bridge]]
    name = "Outbound"
    [bridge.local]
        protocol = "tcp"
            host = "0.0.0.0"
            port = 54321

    [bridge.forward]
        protocol = "fasp"
            host = "GW1"
            port = 54321

[[bridge]]
    name = "Inbound"
    [bridge.local]
        protocol = "fasp"
            host = "0.0.0.0"
            port = 12345

    [bridge.forward]
        protocol = "tcp"
            host = "Server2"
            port = 12345
 ```

##Installation

### Archive bundle (macOS/Linux)

Simply extract the archive: 

* macOS: `tar xvzf ./ibm-faspio-gateway-*-macOS.tar.gz`
* Linux: `tar xvzf ./ibm-faspio-gateway-*-Linux.tar.gz`

A sample `gateway.toml` config file is located under the config folder

Run the gateway manually from the main folder like this: `./bin/faspio-gateway --config config/gateway.toml`

### SystemD Service (Linux)

Install the package: `sudo apt install ./ibm-faspio-gateway_*_amd64.deb`

By default, the config is located at `/usr/local/etc/faspio/gateway.toml`

## Configuration

### Gateway Configuration

**Archive Installations:** 
By default, the faspio Gateway configuration file, gateway.toml, is located in the config directory of the extracted archive:
`./config/gateway.toml`

**SystemD Installations:** 
The faspio Gateway configuration file, gateway.toml, is located in the system `/etc` directory:
`/usr/local/etc/faspio/gateway.toml`

The gateway.toml included in the installation serves as an example. Modify this file to specify your ports, hostnames, and so on.
 
### Logging Configuration
Logging configuration is defined in logging.toml. 
This file is located in the same config directory as gateway.toml (location described above). 
Warning, error, and info logs are sent to the console by default.

In the default logging.toml file, four loggers are made available:
 * `bridge` – High-level logger for the bridge.
 * `s2s` – Stream-to-stream session class logger. 
 * `faspio-cpp` – Logger for the Asio/C++ FASP SDK. 
 * `faspio-c` – Logger for the FASP protocol.
 
For more information on how to configure logging, see the full reference at:
https://github.com/guangie88/spdlog_setup

## Run

* Start service: `systemctl start faspio-gateway`
* Stop service: `systemctl stop faspio-gateway`
* Enable service: `systemctl enable faspio-gateway` (to restart after reboot)
* Service status: `systemctl status faspio-gateway`
* See service logs: `journalctl --unit=faspio-gateway`

# Examples

## Simple echo test through netcat

On the Server machine, listen:

```bash
nc -v -l 12345
```
On the client machine, connect:

```bash
nc <faspio GW1> 12345
```

## Stream data through netcat

On the Server machine, listen:

```bash
nc -v -l 12345 > /dev/null
```
On the client machine, connect and stream 1GB of data:

```bash
dd if=/dev/zero count=1024 bs=1m  | nc -v -n <faspio GW1> 12345 >/dev/null
```
